#include "variables.h"


double Rflux(int Sp,arySp Cv,double t,struct dataProf Met,struct dataInit Init)
/* Flux rate for vapors is defined here to be the 
   (vapor phase sources-sinks)-(vapor deposition)   
   Units are  g/cc   */
{
 double rate,dmsFlux,ohSource,ohMax,koT,khT,Mair, so2Yield;
 arySp Velocity,ohConstant;
ohConstant[dmsSP]=1.2e-11*exp(-260./Met.temp); 
 /*cm3/molec/s OH+DMS, p.32, DeMore et al. (NASA), 1994*/

Mair=(Met.pres/(RGAS*Met.temp))*AVOG/CCperM3;
 /*concentration of air molecules in molec/cm3 */
koT=3.e-31*pow((Met.temp/300.),-3.3); /*low pressure limit at temp */
khT=1.5e-12*pow((Met.temp/300.),0.); /*high pressure limit at temp */
ohConstant[so2SP]=((koT*Mair)/(1.+(koT*Mair/khT)))*
 pow(0.6, (1./pow(1.+(LOG(koT*Mair/khT)),2.)));
 /*cm3/molec/s OH+SO2, p.83, DeMore et al. (NASA), 1990*/

Velocity[so2SP]= 0.005; Velocity[h2so4SP] = 0.01; /*m/s*/
so2Yield=0.8; /*Pandis et al., 1994 */
ohSource=Init.ohestd*AVOG*1.e-9/(RGAS*(Met.temp/Met.pres)*CCperM3);
 /*ppb to molec/cm3*/
/*ohSource=0; ohMax=5.;*/ /* molec/cm3 Russell et al., 1994*/
/*if ((Met.tday>0.25)&&(Met.tday<0.75)&&(Met.rhum < 1.) )
 ohSource=ohMax*sin(Met.tday); *//*Russell et al., 1994 - OR use UV/NOx data */

switch (Sp) 
  {
  case dmsSP:
   dmsFlux=dmsFLUX*perMIL*MolWt(dmsSP)/(HRperDAY*SECperHR*Init.height*CCperM3);
    /* g/cc/s */
    /*Quinn et al., 1990: 0 to 5 umole/m2/d DMS in Central Pacific at 35N */
   rate=dmsFlux -ohConstant[dmsSP]*so2Yield*ohSource*Cv[dmsSP];/*gDMS/cc/s*/
   break;
  case so2SP:
   rate=(ohConstant[dmsSP]*so2Yield*ohSource*Cv[dmsSP]*MolWt(so2SP)/MolWt(dmsSP))
    -(ohConstant[so2SP]*ohSource*Cv[so2SP])
    -((Velocity[Sp]/Init.height)*Cv[Sp]); /*gSO2/cc/s*/
   break;
  case h2so4SP:
   rate=(ohConstant[so2SP]*ohSource*Cv[so2SP]*MolWt(h2so4SP)/MolWt(so2SP))
    -((Velocity[Sp]/Init.height)*Cv[Sp]); /*gH2SO4/cc/s*/
   break;
  default:
   rate =0.0;
   break;
  }
return rate;
}

double Rnucl(int Dp,int Sp, int Tp, arySp Cv, struct dataProf Met)
/* #/cm3/s */
{	
 double TotVol=0,dryNuclMass,totMass,dryDensity,dryCritDiam;
 double PRate=0,rate=0,TotNuclMass=0,CritDiam,WaterMolec,AcidMolec=0,h2so4Nv;
 double rhum;
 arySp NuclTuner;
 NuclTuner[h2so4SP] = 1.e0; /* Nucl.Scal.Fac.=1; JHS, 10/17/96 */
if ((Sp==h2oSP)|(Sp==nh42so4SP)|(Sp==nh4hso4SP)|(Sp==h2so4SP)) {
  if ((Tp==sulfateTP)|(Tp==vaporTP)) {
    if (Cv[h2so4SP]>=h2so4THRESH) {
      h2so4Nv=(Cv[h2so4SP]/(MolWt(h2so4SP)/AVOG))*CCperM3; 
       /*convert g/cc to molec/m3*/
      rhum=Cv[h2oSP]/h2oSatWv(Met.temp);
 h2so4nucleation_(&rhum,&h2so4Nv,&Met.temp,&PRate,  
       &WaterMolec,&AcidMolec);
      PRate/=CCperM3; /*convert p/m3/s to p/cc/s*/}
    dryNuclMass =AcidMolec*(so4hso4FRAC*MolWt(nh42so4SP)
     +(1.-so4hso4FRAC)*MolWt(nh4hso4SP))/AVOG; /* g */
    TotVol=AcidMolec*(so4hso4FRAC*MolWt(nh42so4SP)/SpDensity(nh42so4SP)
     +(1.-so4hso4FRAC)*MolWt(nh4hso4SP)/SpDensity(nh4hso4SP))/AVOG; /*um3*/
    if ((TotVol>0)&&(dryNuclMass>0)) {dryDensity=dryNuclMass/TotVol;} /* g/um3 */
    else {dryDensity=SpDensity(nh42so4SP);}
    dryCritDiam=pow((6.*dryNuclMass/(dryDensity*PI)),(1./3.)); /* um */
    if ((dryCritDiam>dryDp(Dp-0.5))&&(dryCritDiam<dryDp(Dp+0.5))) {
      totMass=wetNuclMass(Cv,Met);
      switch (Sp) 
      {
      case h2oSP:
       if (Tp==sulfateTP) {
        rate=WaterMolec*(MolWt(nh42so4SP)/(AVOG*totMass))
         *NuclTuner[h2so4SP]*PRate/CCperM3;}
       break;
      case nh42so4SP:
       if (Tp==sulfateTP) {
        rate=(so4hso4FRAC)*AcidMolec*(MolWt(nh42so4SP)/(AVOG*totMass))
         *NuclTuner[h2so4SP]*PRate/CCperM3;}
       break;
      case nh4hso4SP:
       if (Tp==sulfateTP) {
        rate=(1.-so4hso4FRAC)*AcidMolec*(MolWt(nh4hso4SP)/(AVOG*totMass))
         *NuclTuner[h2so4SP]*PRate/CCperM3;}
       break;
      case h2so4SP:
       if (Tp==vaporTP) {
        rate=AcidMolec*(MolWt(h2so4SP)/(AVOG*totMass))
         *NuclTuner[h2so4SP]*PRate/CCperM3;}
       break;
      default:
       break;
} } } }

return rate;
}

double wetNuclMass(arySp Cv,struct dataProf Met)
/* Returns the total mass in the critical nucleus (g*)*/
{
 double mass=0, AcidMolec=0,PRate,WaterMolec=0,rhum,h2so4Cv;
if (Cv[h2so4SP]>=h2so4THRESH) {
  rhum=Cv[h2oSP]/h2oSatWv(Met.temp);
  h2so4Cv=(Cv[h2so4SP]/(MolWt(h2so4SP)/AVOG))*CCperM3; /*convert g/cc to molec/m3*/
  h2so4nucleation_(&rhum,&h2so4Cv,&Met.temp,&PRate,&WaterMolec,&AcidMolec);}
mass=(AcidMolec*(so4hso4FRAC*MolWt(nh42so4SP)+(1-so4hso4FRAC)*MolWt(nh4hso4SP))
 +WaterMolec*MolWt(h2oSP))/AVOG;
return mass;
}


double Rcond(int Dp,int Sp,int Tp,int VFlag,aryDpSpTp Mp,arySp Cv,double Np, 
 struct dataProf Met)
/* g/s/cm3
   Calculates the rate of condensation of species Sp onto particles 
   of size Dp.  */
{
 double rate=0,diam,h2so4MassFrac,h2so4SatVp=0,h2so4CvSurface,h2so4Cv;
 double h2oSatVp,h2oCvBulk,h2oCvSurface,rhum;
 double sigmaA,sigmaB,sigmaAB,cB,Z,N,beta,lambda,diffusivity,z,cA,Kn;
 double y,ybig,svw,massbalance,heatbalance;
 double deltaV,deltaT,alphaV,alphaT,rhoa,rhop,cpa,conductivity,Le,Tsurface;
 double CorrDiffusivity, CorrConductivity;
 arySp MeanPath,CRateFactor;
diam=wetDp(Dp,Tp,Mp[Dp][h2oSP][Tp],Mp,Np)/MICperM;
if ((Sp==h2oSP)|(Sp==nh42so4SP)|(Sp==nh4hso4SP)|(Sp==h2so4SP)) {
  if ((Tp==sulfateTP)|(Tp==vaporTP)) {
    if (Cv[h2so4SP]>=h2so4THRESH) {
      CRateFactor[h2so4SP]=0;
      h2so4Cv=(Cv[h2so4SP]/(MolWt(h2so4SP)/AVOG))*CCperM3; 
       /*convert g/cc to molec/m3*/
      rhum=Cv[h2oSP]/h2oSatWv(Met.temp);
      h2so4condensation_(&rhum,&h2so4Cv,&Met.temp,&diam,
       &CRateFactor[h2so4SP],&h2so4MassFrac,&h2so4SatVp);
      h2so4CvSurface=Activity(h2so4SP,Tp,Dp,Mp[Dp][h2oSP][Tp],Mp,Np,Met)
       *(h2so4SatVp*NM2perTORR
       /NM2perMBAR)*MolWt(h2oSP)/(RGAS*Met.temp*CCperM3);
      switch (Sp) 
      {
      case h2oSP:
       if (!(VFlag==vaporTP)) {
        rate+=((1.-h2so4MassFrac)
         *CRateFactor[h2so4SP]*(Cv[h2so4SP]-h2so4CvSurface)*Np);}
       break;
      case nh42so4SP:
       if (!(VFlag==vaporTP)) {
        rate+=(so4hso4FRAC*h2so4MassFrac*(MolWt(nh42so4SP)/MolWt(h2so4SP))
         *CRateFactor[h2so4SP]*(Cv[h2so4SP]-h2so4CvSurface)*Np);}
       break;
      case nh4hso4SP:
       if (!(VFlag==vaporTP)) {
        rate+=((1.-so4hso4FRAC)*h2so4MassFrac*(MolWt(nh4hso4SP)/MolWt(h2so4SP))
         *CRateFactor[h2so4SP]*(Cv[h2so4SP]-h2so4CvSurface)*Np);}
       break;
      case h2so4SP:
       if (VFlag==vaporTP) {
        rate+= (h2so4MassFrac
         *CRateFactor[h2so4SP]*(Cv[h2so4SP]-h2so4CvSurface)*Np);} 
       break;
      default:
       rate+=0.0;
       break;  }
/*printf("H2SO4 diam%e Act%e Cv%e CvSurf%e SatVp%e CRate%e rate%e Np%e \n",
 diam,Activity(h2so4SP,Tp,Dp,Mp,Met),Cv[h2so4SP],h2so4CvSurface,h2so4SatVp,
 CRateFactor[h2so4SP],rate,Np);*/
} } }
if ((Sp==nh42so4SP)|(Sp==nh4hso4SP)|(Sp==so2SP)) {
  sigmaA=pow((0.58/PI),0.5)/NMperM;/* m Atkins, p. 831*/
  cB=pow((8.*kBOLTZ*Met.temp/(PI*(MolWt(airSP)/(AVOG*GperKG)))),0.5); /*m/sec*/
  Z=6.9189e9; /* Collision freq. sec-1 at 1 atm CRC p. 14-13 */
  N=2.5470e25; /* Number density m-3 at 1 atm CRC p. 14-13 */
  sigmaB=pow((0.43/PI),0.5)/NMperM;/* m Atkins, for N2, p. 831*/
  sigmaAB=(sigmaA+sigmaB)/2.; /*m */
  z=MolWt(so2SP)/MolWt(airSP); /* Seinfeld, 1986, p.310 */
  diffusivity=(3./(8.*PI*(Met.pres*NM2perMBAR/(kBOLTZ*Met.temp))*pow(sigmaAB,2.)))
   *pow((PI*kBOLTZ*Met.temp*(1+z)/(2*MolWt(so2SP)/(AVOG*GperKG))), 0.5);
   /* m2/sec Seinfeld, 1986, p. 311 */
  cA=pow((8.*kBOLTZ*Met.temp/(PI*(MolWt(so2SP)/(AVOG*GperKG)))),0.5); /*p.311*/
  lambda=diffusivity*32./(3.*PI*(1+z)*cA);  /*m eqn. 8.9 p.311 */
  Kn=2.*lambda/(diam);
  beta=(1.+Kn)/(1.+(1.71*Kn)+(1.33*Kn*Kn)); /* eqn 8.111, p. 337 */
  CRateFactor[so2SP]=2.*PI*diam*diffusivity*beta*CCperM3;
/*all condensed SO2 oxidizes (Pandis et al., 1994)*/
  switch (Sp) 
  {
  case nh42so4SP:
   if (!(VFlag==vaporTP)) {
    rate+=(so4hso4FRAC*(MolWt(nh42so4SP)/MolWt(so2SP))
     *CRateFactor[so2SP]*(Cv[so2SP])*Np);}
   break;
  case nh4hso4SP:
   if (!(VFlag==vaporTP)) {
    rate+=( (1.-so4hso4FRAC)*(MolWt(nh4hso4SP)/MolWt(so2SP))
     *CRateFactor[so2SP]*(Cv[so2SP])*Np );}
   break;
  case so2SP:
   if (VFlag==vaporTP) {
    rate+=( CRateFactor[so2SP]*(Cv[so2SP])*Np );} 
  default:
   rate+=0.0;
   break;  
  }
/*printf("SO2 diam%e dif%e Cv%e beta%e sig%e CRate%e rate%e Np%e\n",
 diam,diffusivity,Cv[so2SP],beta,sigmaAB,
 CRateFactor[so2SP],rate,Np);*/
}
if (Sp==h2oSP){
	if(Cv[h2oSP]>h2oSatWv(Met.temp)){
diffusivity=0.211*pow((Met.temp/ZEROC),1.94)*(1013.25/Met.pres)/pow(CMperM,2.);
   /*P&K, p. 413, cm2/s converted to m2/s*/
deltaV=1.3*8.e-6/CMperM;/*p.415 P&K*/
deltaT=2.16e-5/CMperM;/*p.415 P&K */
alphaV=1.0;/*JHS, pers. comm.*/
alphaT=1.0; /*JHS, pers. comm.*/
rhop=wetPDensity(Dp,Tp,Mp[Dp][h2oSP][Tp],Mp)*pow(CMperMIC,-3.0); /*g/cc*/
rhoa=Met.pres*(MolWt(airSP)/GperKG)/(RGAS*Met.temp); /*kg/m3*/
cpa=1.007;/*J/g/K, at 27degC, CRCp.6-2*/
conductivity=(5.69+0.017*(Met.temp-ZEROC))*1.e-5*JperCAL*CMperM;
/*cal/cm/sec/C converted to J/m/s/C= J/msK*/
CorrConductivity=conductivity/((diam/(diam+2.*deltaT))  
 +(conductivity*2./(diam*alphaT*rhoa*GperKG*cpa))   
 *pow((2.*PI*(MolWt(airSP)/GperKG)/(RGAS*NM2perMBAR*Met.temp)),0.5));
Le=597.3*pow((ZEROC/Met.temp),(0.167+3.67e-4*Met.temp))*JperCAL; 
/*ITcal/g -> J/g specific latent heat from Pruppacher and Klett*/  
CorrDiffusivity=diffusivity/((diam/(diam+2.*deltaV))
 +(diffusivity*2./(diam*alphaV))
 *pow((2.*PI*(MolWt(airSP)/GperKG)/(RGAS*NM2perMBAR*Met.temp)),0.5));
CRateFactor[h2oSP]=2.*PI*diam*CorrDiffusivity*CCperM3;
h2oCvBulk=Cv[h2oSP];
y=Surface(Dp,Tp,Mp[Dp][h2oSP][Tp],Mp,Np,Met)
 -Activity(h2oSP,Tp,Dp,Mp[Dp][h2oSP][Tp],Mp,Np,Met);
/*if (y>0.05) {ybig=exp(y); y=ybig-1.0;}*/
svw=(h2oCvBulk/h2oSatWv(Met.temp)) -1.0;
massbalance=(rhop)/(CorrDiffusivity*h2oSatWv(Met.temp)/*missing terms (MolWt(h2oSP),Rgas and Temp) are all taken care of in h2oSatWv(met.temp)*/);
heatbalance=(Le*rhop*pow(CMperM,3.0)/(CorrConductivity*Met.temp))
 *((Le*MolWt(h2oSP)/(RGAS*Met.temp*NM2perMBAR))-1.0);
rate+=(2.0*PI*diam*rhop*pow(CMperM,3.0)*(svw-y)*Np/(massbalance+heatbalance));/*(equ 13-38 p&k 1978)*/
printf("rate testing Dp %d svw%e y%e massbalance%e heatbalance%e\n",Dp, svw, y, massbalance, heatbalance );
printf("h2oCvBulk%e h2oSatWv%e surface%e activity%e rate%e\n", h2oCvBulk, h2oSatWv(Met.temp), 
	Surface(Dp,Tp,Mp[Dp][h2oSP][Tp],Mp,Np,Met), Activity(h2oSP,Tp,Dp,Mp[Dp][h2oSP][Tp],Mp,Np,Met), rate );


/*rate+=2.0*PI*diam*rhop*pow(CMperM,3.0)*(1+svw-exp(y))*Np/(massbalance+heatbalance);*/
/*h2oCvSurface=Surface*Activity(h2oSP,Tp,Dp,Mp,Met)*h2oSatWv(Tsurface);*/
/*rate+=1.*CRateFactor[h2oSP]*(h2oCvBulk- h2oCvSurface)*Np;*/
/*  rate+=1.*CRateFactor[h2oSP]*(h2oCvBulk-0.)*Np;*/
}}
  return rate;
}


double Rdepn(int Dp,int Sp,int Tp,aryDpSpTp Mp,double Np,struct dataProf Met,
 struct dataInit Init)
/* (particles)/cm3/s:  
Deposition is calculated by the ratio of the terminal settling velocity 
(Seinfeld, 1986, p. 317-319, eqn 8.36 & 8.41) to the boundary layer height. */
{
 double rate=0,DepVelocity,Cc;
if (wetSMass(Dp,Tp,Mp[Dp][h2oSP][Tp],Mp)>0) {
Cc=1.+(2.*AirMeanPath(Met)/wetDp(Dp,Tp,Mp[Dp][h2oSP][Tp],Mp,Np))
 *(1.257+0.4*exp(-1.1*wetDp(Dp,Tp,Mp[Dp][h2oSP][Tp],Mp,Np)/(2.*AirMeanPath(Met))));
DepVelocity=wetPMass(Dp,Tp,Mp[Dp][h2oSP][Tp],Mp,Np)*GRAVITY*Cc
 /(3.*PI*AirViscosity(Met)*wetDp(Dp,Tp,Mp[Dp][h2oSP][Tp],Mp,Np));
rate =(Mp[Dp][Sp][Tp]/wetSMass(Dp,Tp,Mp[Dp][h2oSP][Tp],Mp))*(DepVelocity/Init.height)*Np;}
return rate;
}

double RcoagAdd(int Dp,int Sp,int Tp,aryDpSpTp Mp,aryDpTp Np,struct dataProf Met)
{
/*The number of collisions is calculated for each particle population type k.
   Add: collisions with any sizes such that dryDpi-.5<dryDp1+dryDp2<dryDpi+.5
  This gives the number to or from this section.  This is then converted
  to the mass of species j in type k by multiplying by the fraction present. 
  number added to each section is returned. */
 int i1,i2,k1,k2,j;
 double AddNum=0,TotalNum=0,AddSpMass=0,TotalMass=0,rate=0,sumDp;
for (i1=0;i1<=Dp;i1++) {
  for(k1=0; k1<TpDIM;k1++)  {if ((Np[i1][k1]>0)&&(wetSMass(i1,k1,Mp[Dp][h2oSP][Tp],Mp)>0)) {
    for(k2=0; k2<TpDIM;k2++)  {
      if (min(k1,k2)==Tp) {
        for (i2=0;i2<=Dp;i2++) {if ((Np[i2][k2]>0)&&(wetSMass(i2,k2,Mp[Dp][h2oSP][Tp],Mp)>0)) {
          sumDp=pow( ((dryPMass(i1,k1,Mp)+dryPMass(i2,k2,Mp))
           /(PI*dryPDensity(Dp,Tp,Mp)/6.)), (1./3.) ); 
          if ((sumDp>dryDp(Dp-0.5))&&(sumDp<dryDp(Dp+0.5))) {         
            AddNum=0.5*CoagCoef(wetDp(i1,k1,Mp[i1][h2oSP][k1],Mp,Np[i1][k1]),
             wetPMass(i1,k1,Mp[i1][h2oSP][k1],Mp,Np[i1][k1]),
             wetDp(i2,k2,Mp[i2][h2oSP][k2],Mp,Np[i2][k2]),
             wetPMass(i2,k2,Mp[i2][h2oSP][k2],Mp,Np[i2][k2]),Met)
             *Np[i1][k1]*Np[i2][k2];
            TotalNum+=AddNum;
            AddSpMass+=((Mp[i1][Sp][k1]/wetSMass(i1,k1,Mp[i1][h2oSP][k1],Mp))
             *wetPMass(i1,k1,Mp[i1][h2oSP][k1],Mp,Np[i1][k1])*AddNum);
            for (j=0; j<SpDIM;j++) {
             TotalMass+=((Mp[i1][j][k1]/wetSMass(i1,k1,Mp[i1][h2oSP][k1],Mp))
              *wetPMass(i1,k1,Mp[i1][h2oSP][k1],Mp,Np[i1][k1])*AddNum);}
            AddSpMass+=((Mp[i2][Sp][k2]/wetSMass(i2,k2,Mp[i2][h2oSP][k2],Mp))
             *wetPMass(i2,k2,Mp[i2][h2oSP][k2],Mp,Np[i2][k2])*AddNum);
            for (j=0; j<SpDIM;j++) {
             TotalMass+=((Mp[i1][j][k1]/wetSMass(i1,k1,Mp[i1][h2oSP][k1],Mp))
              *wetPMass(i1,k1,Mp[i1][h2oSP][k1],Mp,Np[i1][k1])*AddNum);}
} }}} } } }}
if (TotalMass>0) {rate= (AddSpMass/TotalMass)*TotalNum;}
return rate;
}

double RcoagSub(int Dp,int Sp,int Tp,aryDpSpTp Mp,aryDpTp Np,struct dataProf Met)
{
/*The number of collisions is calculated for each particle population type k.
   Subtract: collisions with all sizes
  This gives the number to or from this section.  This is then converted
  to the mass of species j in type k by multiplying by the fraction present. 
  Number subtracted from each section is returned. */
 int i1,k1;
 double SubtNum=0, rate=0;
if ((Np[Dp][Tp]>0)&&(wetSMass(Dp,Tp,Mp[Dp][h2oSP][Tp],Mp)>0)) {
  for (i1=0;i1<DpDIM;i1++) {
    for(k1=0; k1<TpDIM;k1++)  {  if (Np[i1][k1]>0) {   
      SubtNum+=(CoagCoef(wetDp(i1,k1,Mp[i1][h2oSP][k1],Mp,Np[i1][k1]),
       wetPMass(i1,k1,Mp[i1][h2oSP][k1],Mp,Np[i1][k1]),
       wetDp(Dp,Tp,Mp[Dp][h2oSP][Tp],Mp,Np[Dp][Tp]),
       wetPMass(Dp,Tp,Mp[Dp][h2oSP][Tp],Mp,Np[Dp][Tp]),Met)
       *Np[i1][k1]*Np[Dp][Tp]); 
/*printf("Coag i%dk%d %e %e %e %e\n",Dp,Tp,Np[Dp][Tp],Np[i1][k1],SubtNum,
       CoagCoef(wetDp(i1,k1,Mp[i1][h2oSP][k1],Mp,Np[i1][k1]),
       wetPMass(i1,k1,Mp[i1][h2oSP][k1],Mp,Np[i1][k1]),
       wetDp(Dp,Tp,Mp[Dp][h2oSP][Tp],Mp,Np[Dp][Tp]),
       wetPMass(Dp,Tp,Mp[Dp][h2oSP][Tp],Mp,Np[Dp][Tp]),Met) );*/
}}}
rate = (Mp[Dp][Sp][Tp]/wetSMass(Dp,Tp,Mp[Dp][h2oSP][Tp],Mp))*SubtNum;}
/*printf("CoagRate i%dk%d %e %e %e\n",Dp,Tp,rate,Np[Dp][Tp],SubtNum);*/
return rate;
}


double CoagCoef(double wetDp1, double mass1, double wetDp2, double mass2, 
 struct dataProf Met)
/*Coagulation coefficients are calculated from Seinfeld, 1986, p. 397
 The units are cm3/sec and this calculation is done in cgs.*/
{
 double coef=0,diam1,diam2,c1,c2,c12,Kn1,Kn2,l1,l2,g1,g2,g12,D1,D2;
if (!((mass1<=zeroTHRESH)|(mass2<=zeroTHRESH))) {
  diam1=wetDp1*CMperMIC;  diam2=wetDp2*CMperMIC;  
  Kn1=2.*AirMeanPath(Met)/diam1;  Kn2=2.*AirMeanPath(Met)/diam2;
  c1=pow((8.*kBOLTZ*ktoCGS*Met.temp/(PI*mass1)),0.5); 
  c2=pow((8.*kBOLTZ*ktoCGS*Met.temp/(PI*mass2)),0.5); 
  c12=pow((pow(c1,2.)+pow(c2,2.)),0.5);
  D1=(kBOLTZ*ktoCGS*Met.temp/(3*PI*AirViscosity(Met)*diam1)) 
   *((5.+4.*Kn1+6.*pow(Kn1,2.)+18.*pow(Kn1,3.))/(5.-Kn1+(8.+PI)*pow(Kn1,2.)));
  D2=(kBOLTZ*ktoCGS*Met.temp/(3*PI*AirViscosity(Met)*diam2)) 
   *((5.+4.*Kn2+6.*pow(Kn2,2.)+18.*pow(Kn2,3.))/(5.-Kn2+(8.+PI)*pow(Kn2,2.)));
  l1=8.*D1/(PI*c1);   l2=8.*D2/(PI*c2);   
  g1=(1./(3.*diam1*l1))*(pow((diam1+l1),3.)-pow((pow(diam1,2.)
   +pow(l1,2.)),1.5))-diam1;
  g2=(1./(3.*diam2*l2))*(pow((diam2+l2),3.)-pow((pow(diam2,2.)
    +pow(l2,2.)),1.5))-diam2;
  g12=pow((pow(g1,2.)+pow(g2,2.)),0.5);
  coef=2.*PI*(D1+D2)*(diam1+diam2)
   /( ((diam1+diam2)/(diam1+diam2+2.*g12))+((8.*(D1+D2))/(c12*(diam1+diam2))));}
if (coef>1.e20) {printf("Coef: d%e %e D%e %e g%e %e %e C%e\n",diam1,diam2,D1,D2,
g1,g2,g12,coef);}
return coef;
}


double Rsalt(int Dp,int Sp,int Tp,aryDp SaltNp,struct dataProf Met, struct dataInit Init)
/*Wind dependence of sea spray concentration taken from data of 
Bigg et al. 1995 , Table 2, S winds, CCN @ 0.1% */
{
 double rate=0,C;
 double diam,pmass,naclmass,DepVelocity,Cc,h2oFrac;
if (Tp==saltTP){ if ((Sp==naclSP)|(Sp==h2oSP)) {
  C=pow(10.,(0.48+0.043*Init.wind));
  if (SaltNp[Dp]>0) {
    h2oFrac=1.-naclFRAC; /*neglect Ca, Fl, etc. */
    naclmass=PI*SpDensity(naclSP)*pow(dryDp(Dp),3.)/6.;
    pmass=(1./naclFRAC)*naclmass;/*g/particle*/
    diam=pow(((6./PI)*(pmass/SeaSaltDensity())),(1./3.)); /*um*/
    Cc=1.+(2.*AirMeanPath(Met)/diam)
     *(1.257+0.4*exp(-1.1*(diam/2.)/AirMeanPath(Met)));
    DepVelocity=(pmass)*GRAVITY*Cc/(3.*PI*AirViscosity(Met)*diam);
    rate =(DepVelocity/Init.height)*SaltNp[Dp]*C;}
  if (Sp==naclSP) {rate *= naclFRAC;}
  if (Sp==h2oSP) {rate *= h2oFrac;}   }}
return rate;
}


double Rcomb(int Sp,int Dp)
{
 double rate;
if (Sp==1)  rate = 0.0;
else rate =0;
return rate;
}



double Rgrow(aryDpSpTp Mp,aryDpTp Np)
/* Calculates the growth of particles from section Dp to section Dp+1 for particles of type Tp with component species Sp.  Inputs are the calculated changes in number (dNp) and mass (dMp) from nucleation, condesation, coagulation, and composition.  There are two processes which result in what we define here to be "growth."  These are the two processes whereby particles must be added to a bin in which they don't necessarily "fit."  In both cases, both number and mass are conserved for the sum of bins Dp and Dp+1, and the number of particles grown is uniquely constrained by those conditions (and by assuming each particle bin is monodisperse. 
The values out put in this routine are the mass and number in the bins FROM which growth occurs (Mp[Dp][Sp][Tp] and Np[Dp][Tp]) and TO which growth occurs (Mp[Dp+1][Sp][Tp] and Np[Dp+1][Tp]).  Values of current Mp and Np include the additions from rates (i.e. dNpdt*dt, and dMpdt*dt) -- that is, this summation is done before this function is called.  This is to ensure that the composition of the grown particles reflects any mass added to particular species by condensation.  The composition of the new particles in section i is assumed to be the same as the old particles in that section. The composition of the new particles in section i+1 is the remaining mass so that species which increase in composition are preferentially allocated to the larger (i+1) section, but no mass is lost.  The rate of growth is also returned, in units of particles/dt.*/
{
 double rate,dPMi,dPMi1,dSMi,dSMi1,newNpi,newMpi,Pgrown;
 FILE            *GrowthFilePtr;
 word24          GrowthFileName;
 int i,j,k;
strcpy(GrowthFileName,"Growth");
strcat(GrowthFileName,OUTsuffix);
for (k=0;k<TpDIM;k++) {
  for (i=0;i<DpDIM-1;i++) {
    dPMi=dryPMass(i,k,Mp);
    dPMi1=dryPMass(i+1,k,Mp);
    dSMi=drySMass(i,k,Mp);
    dSMi1=drySMass(i+1,k,Mp);
   if (((dSMi/dPMi)>(Np[i][k]+zeroTHRESH))
   || ((dSMi1/dPMi1)<(Np[i+1][k]+zeroTHRESH))) {
    newNpi=(dPMi1/(dPMi1-dPMi))*(Np[i][k]+Np[i+1][k]);
    for (j=0;j<SpDIM;j++){if (!(j==h2oSP)) {
      newNpi-=( (1./(dPMi1-dPMi))*(Mp[i][j][k] +Mp[i+1][j][k]) ); }} }
   else {newNpi=Np[i][k];}
    if (newNpi<0.) {newNpi=0.;}          /*particles in i bigger than i+1*/
    if (newNpi>(Np[i][k]+Np[i+1][k])) {  /*particles in i+1 smaller than i*/
      newNpi=Np[i][k]+Np[i+1][k];   }
    Pgrown=Np[i][k]-newNpi;

    for (j=0;j<SpDIM;j++){ 
        if ((Pgrown>zeroTHRESH)&&(Mp[i][j][k]>0)) {           /*i grow to i+1*/
         newMpi=Mp[i][j][k]-(Pgrown*Mp[i][j][k]*(dPMi1/dSMi));}
        else {                                                /*i+1 evap to i*/
          if ((Pgrown<-zeroTHRESH)&&(Mp[i+1][j][k]>0)) {
            newMpi=Mp[i][j][k]-(Pgrown*Mp[i+1][j][k]*(dPMi/dSMi1));   }
          else {newMpi=Mp[i][j][k];     /*no growth*/ }}           
      if (newMpi>(Mp[i+1][j][k]+Mp[i][j][k])) {newMpi=Mp[i+1][j][k]+Mp[i][j][k];}
      if (newMpi<0.)   {newMpi=0.;} 
      if (newNpi==0.) {newMpi=0.0;}
      Mp[i+1][j][k]+=(Mp[i][j][k]-newMpi);     
      Mp[i][j][k]=newMpi;} /*end of j loop*/
GrowthFilePtr=fopen(GrowthFileName,"a");
fprintf(GrowthFilePtr,"i%dk%d Np%e eNp%e Pg%e Np'%e eNp%e\n",i,k,Np[i][k],(dSMi/dPMi),
Pgrown,newNpi,(drySMass(i,k,Mp)/dryPMass(i,k,Mp)));
fclose(GrowthFilePtr);
    Np[i+1][k]+=(Np[i][k]-newNpi);
    Np[i][k]=newNpi;
} }/*end of i,k loop*/
return Pgrown;
}
